using Microsoft.AspNetCore.Mvc;
using OneReview.Domain;
using OneReview.Services;
using System.Collections;
using System.Collections.ObjectModel;

namespace OneReview.Controllers;

[ApiController]
[Route("[controller]")]
public class ProductsController(ProductService productService) : ControllerBase
{

    private readonly ProductService _ProductService = productService;

    [HttpGet()]
    public IActionResult Get()
    {
        ReadOnlyCollection<Product> products = _ProductService.Get();
        ProductResponse?[] productResponses = ProductResponse.FromDomain(products);
        return Ok(productResponses);
    }

    [HttpPost]
    public IActionResult Create(CreateProductRequest request)
    {
        // mapping to internal representation
        Product product = request.ToDomain();

        // invoke the use case
        _ProductService.Create(product);

        // mapping to external representation
        ProductResponse productResponse = ProductResponse.FromDomain(product);

        // return 201 created response
        return CreatedAtAction(
            actionName: nameof(Get),
            routeValues: new { ProductId = product.Id },
            value: productResponse);
    }

    [HttpGet("{productId:guid}")]
    public IActionResult Get(Guid productId)
    {
        //get the product
        Product? product = _ProductService.Get(productId);

        // mapping to external representation
        ProductResponse? productResponse = ProductResponse.FromDomain(product);

        // return 200 ok response
        return productResponse is null
            ? Problem(statusCode: StatusCodes.Status404NotFound, detail: $"Product not found {productId}")
            : Ok(productResponse);
    }

    public record CreateProductRequest(
        string Name,
        string Category,
        string SubCategory)
    {

        public Product ToDomain() =>
            new()
            {
                Name = Name,
                Category = Category,
                SubCategory = SubCategory,
            };

    }

    public record ProductResponse(
        Guid Id,
        string Name,
        string Category,
        string SubCategory)
    {

        public static ProductResponse? FromDomain(Product? product) =>
            product is null ? null : new ProductResponse(
                Id: product.Id,
                Name: product.Name,
                Category: product.Category,
                SubCategory: product.SubCategory);

        public static ProductResponse?[] FromDomain(IEnumerable<Product> products) =>
            products.Select(FromDomain).ToArray();

    }

}